@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- * libmm -- Access modem status & information from glib applications
+ * libmm-glib -- Access modem status & information from glib applications
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #include <ModemManager.h>
 
 #include "mm-helpers.h"
+#include "mm-common-helpers.h"
 #include "mm-errors-types.h"
 #include "mm-gdbus-manager.h"
 #include "mm-manager.h"
@@ -65,19 +66,19 @@ get_proxy_type (GDBusObjectManagerClient *manager,
 
     if (g_once_init_enter (&once_init_value)) {
         lookup_hash = g_hash_table_new (g_str_hash, g_str_equal);
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem",                GSIZE_TO_POINTER (MM_TYPE_MODEM));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Messaging",      GSIZE_TO_POINTER (MM_TYPE_MODEM_MESSAGING));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Voice",          GSIZE_TO_POINTER (MM_TYPE_MODEM_VOICE));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Location",       GSIZE_TO_POINTER (MM_TYPE_MODEM_LOCATION));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Time",           GSIZE_TO_POINTER (MM_TYPE_MODEM_TIME));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Signal",         GSIZE_TO_POINTER (MM_TYPE_MODEM_SIGNAL));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Firmware",       GSIZE_TO_POINTER (MM_TYPE_MODEM_FIRMWARE));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Oma",            GSIZE_TO_POINTER (MM_TYPE_MODEM_OMA));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.ModemCdma",      GSIZE_TO_POINTER (MM_TYPE_MODEM_CDMA));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Modem3gpp",      GSIZE_TO_POINTER (MM_TYPE_MODEM_3GPP));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Modem3gpp.Ussd", GSIZE_TO_POINTER (MM_TYPE_MODEM_3GPP_USSD));
-        g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Simple",         GSIZE_TO_POINTER (MM_TYPE_MODEM_SIMPLE));
-        /* g_hash_table_insert (lookup_hash, "org.freedesktop.ModemManager1.Modem.Contacts",    GSIZE_TO_POINTER (MM_GDBUS_TYPE_MODEM_CONTACTS_PROXY)); */
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem",                          GSIZE_TO_POINTER (MM_TYPE_MODEM));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Messaging",                GSIZE_TO_POINTER (MM_TYPE_MODEM_MESSAGING));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Voice",                    GSIZE_TO_POINTER (MM_TYPE_MODEM_VOICE));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Location",                 GSIZE_TO_POINTER (MM_TYPE_MODEM_LOCATION));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Time",                     GSIZE_TO_POINTER (MM_TYPE_MODEM_TIME));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Signal",                   GSIZE_TO_POINTER (MM_TYPE_MODEM_SIGNAL));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Firmware",                 GSIZE_TO_POINTER (MM_TYPE_MODEM_FIRMWARE));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Oma",                      GSIZE_TO_POINTER (MM_TYPE_MODEM_OMA));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.ModemCdma",                GSIZE_TO_POINTER (MM_TYPE_MODEM_CDMA));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Modem3gpp",                GSIZE_TO_POINTER (MM_TYPE_MODEM_3GPP));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Modem3gpp.ProfileManager", GSIZE_TO_POINTER (MM_TYPE_MODEM_3GPP_PROFILE_MANAGER));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Modem3gpp.Ussd",           GSIZE_TO_POINTER (MM_TYPE_MODEM_3GPP_USSD));
+        g_hash_table_insert (lookup_hash, (gpointer) "org.freedesktop.ModemManager1.Modem.Simple",                   GSIZE_TO_POINTER (MM_TYPE_MODEM_SIMPLE));
         g_once_init_leave (&once_init_value, 1);
     }
 
@@ -959,27 +960,10 @@ mm_manager_uninhibit_device_sync (MMManager     *manager,
 
 /*****************************************************************************/
 
-static void
-register_dbus_errors (void)
-{
-  static volatile guint32 aux = 0;
-
-  if (aux)
-      return;
-
-  /* Register all known own errors */
-  aux |= MM_CORE_ERROR;
-  aux |= MM_MOBILE_EQUIPMENT_ERROR;
-  aux |= MM_CONNECTION_ERROR;
-  aux |= MM_SERIAL_ERROR;
-  aux |= MM_MESSAGE_ERROR;
-  aux |= MM_CDMA_ACTIVATION_ERROR;
-}
-
 static void
 mm_manager_init (MMManager *manager)
 {
-    register_dbus_errors ();
+    mm_common_register_errors ();
 
     /* Setup private data */
     manager->priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
