@@ -77,7 +77,7 @@ mm_plugin_create (void)
 
     return MM_PLUGIN (
         g_object_new (MM_TYPE_PLUGIN_ALTAIR_LTE,
-                      MM_PLUGIN_NAME,                "Altair LTE",
+                      MM_PLUGIN_NAME,                MM_MODULE_NAME,
                       MM_PLUGIN_ALLOWED_SUBSYSTEMS,  subsystems,
                       MM_PLUGIN_ALLOWED_PRODUCT_IDS, products,
                       MM_PLUGIN_CUSTOM_AT_PROBE,     custom_at_probe,
