@@ -29,6 +29,7 @@
 #include "mm-errors-types.h"
 #include "mm-modem-helpers-cinterion.h"
 #include "mm-modem-helpers.h"
+#include "mm-port-serial-at.h"
 
 /* Setup relationship between the 3G band bitmask in the modem and the bitmask
  * in ModemManager. */
@@ -216,20 +217,34 @@ parse_bands (guint                      bandlist,
 }
 
 static guint
-take_and_convert_from_matched_string (gchar                  *str,
-                                      MMModemCharset          charset,
-                                      MMCinterionModemFamily  modem_family)
+take_and_convert_from_matched_string (gchar                   *str,
+                                      MMModemCharset           charset,
+                                      MMCinterionModemFamily   modem_family,
+                                      GError                 **error)
 {
-    guint val = 0;
+    guint             val = 0;
+    g_autofree gchar *utf8 = NULL;
+    g_autofree gchar *taken_str = str;
 
-    if (!str)
+    if (!taken_str) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_INVALID_ARGS,
+                     "Couldn't convert to integer number: no input string");
         return 0;
+    }
 
-    if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT)
-        str = mm_charset_take_and_convert_to_utf8 (str, charset);
+    if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT) {
+        utf8 = mm_modem_charset_str_to_utf8 (taken_str, -1, charset, FALSE, error);
+        if (!utf8) {
+            g_prefix_error (error, "Couldn't convert to integer number: ");
+            return 0;
+        }
+    }
 
-    mm_get_uint_from_hex_str (str, &val);
-    g_free (str);
+    if (!mm_get_uint_from_hex_str (utf8 ? utf8 : taken_str, &val)) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Couldn't convert to integer number: wrong hex encoding: %s", utf8 ? utf8 : taken_str);
+        return 0;
+    }
 
     return val;
 }
@@ -291,12 +306,16 @@ mm_cinterion_parse_scfg_test (const gchar                 *response,
         goto finish;
     }
 
-    r2 = g_regex_new ("\\^SCFG:\\s*\"Radio/Band/([234]G)\",\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\)(,*\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\))?",
+    r2 = g_regex_new ("\\^SCFG:\\s*\"Radio/Band/([234]G)\","
+                      "\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\)"
+                      "(,*\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\))?",
                      0, 0, NULL);
     g_assert (r2 != NULL);
+
     g_regex_match_full (r2, response, strlen (response), 0, 0, &match_info2, &inner_error);
     if (inner_error)
         goto finish;
+
     while (g_match_info_matches (match_info2)) {
         g_autofree gchar *techstr = NULL;
         guint             maxband;
@@ -305,16 +324,28 @@ mm_cinterion_parse_scfg_test (const gchar                 *response,
 
         techstr = mm_get_string_unquoted_from_match_info (match_info2, 1);
         if (g_strcmp0 (techstr, "2G") == 0) {
-            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3), charset, modem_family);
+            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3),
+                                                            charset, modem_family, &inner_error);
+            if (inner_error)
+                break;
             parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_GSM, modem_family);
         } else if (g_strcmp0 (techstr, "3G") == 0) {
-            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3), charset, modem_family);
+            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3),
+                                                            charset, modem_family, &inner_error);
+            if (inner_error)
+                break;
             parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_UMTS, modem_family);
         } else if (g_strcmp0 (techstr, "4G") == 0) {
-            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3), charset, modem_family);
+            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3),
+                                                            charset, modem_family, &inner_error);
+            if (inner_error)
+                break;
             parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_LTE_LOW, modem_family);
             if (modem_family == MM_CINTERION_MODEM_FAMILY_DEFAULT) {
-                maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 6), charset, modem_family);
+                maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 6),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_LTE_HIGH, modem_family);
             }
         } else {
@@ -406,9 +437,11 @@ mm_cinterion_parse_scfg_response (const gchar                  *response,
     if (format == MM_CINTERION_RADIO_BAND_FORMAT_SINGLE) {
         r = g_regex_new ("\\^SCFG:\\s*\"Radio/Band\",\\s*\"?([0-9a-fA-F]*)\"?", 0, 0, NULL);
         g_assert (r != NULL);
+
         g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, &inner_error);
         if (inner_error)
             goto finish;
+
         if (g_match_info_matches (match_info)) {
             g_autofree gchar *currentstr = NULL;
             guint current = 0;
@@ -437,26 +470,40 @@ mm_cinterion_parse_scfg_response (const gchar                  *response,
         r = g_regex_new ("\\^SCFG:\\s*\"Radio/Band/([234]G)\",\"?([0-9A-Fa-fx]*)\"?,?\"?([0-9A-Fa-fx]*)?\"?",
                          0, 0, NULL);
         g_assert (r != NULL);
+
         g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, &inner_error);
         if (inner_error)
             goto finish;
+
         while (g_match_info_matches (match_info)) {
             g_autofree gchar *techstr = NULL;
             guint current;
 
             techstr = mm_get_string_unquoted_from_match_info (match_info, 1);
-            if (g_strcmp0(techstr, "2G") == 0) {
-                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2), charset, modem_family);
+            if (g_strcmp0 (techstr, "2G") == 0) {
+                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_GSM, modem_family);
 
-            } else if (g_strcmp0(techstr, "3G") == 0) {
-                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2), charset, modem_family);
+            } else if (g_strcmp0 (techstr, "3G") == 0) {
+                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_UMTS, modem_family);
-            } else if (g_strcmp0(techstr, "4G") == 0) {
-                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2), charset, modem_family);
+            } else if (g_strcmp0 (techstr, "4G") == 0) {
+                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_LTE_LOW, modem_family);
                 if (modem_family == MM_CINTERION_MODEM_FAMILY_DEFAULT) {
-                    current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 3), charset, modem_family);
+                    current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 3),
+                                                                    charset, modem_family, &inner_error);
+                    if (inner_error)
+                        break;
                     parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_LTE_HIGH, modem_family);
                 }
             } else {
@@ -845,22 +892,71 @@ mm_cinterion_parse_swwan_response (const gchar  *response,
     return status;
 }
 
+/*****************************************************************************/
+/* ^SGAUTH response parser */
+
+/*   at^sgauth?
+ *   ^SGAUTH: 1,2,"vf"
+ *   ^SGAUTH: 3,0,""
+ *   ^SGAUTH: 4,0
+ *
+ *   OK
+ */
+
+gboolean
+mm_cinterion_parse_sgauth_response (const gchar          *response,
+                                    guint                 cid,
+                                    MMBearerAllowedAuth  *out_auth,
+                                    gchar               **out_username,
+                                    GError              **error)
+{
+    g_autoptr(GRegex)     r = NULL;
+    g_autoptr(GMatchInfo) match_info = NULL;
+
+    r = g_regex_new ("\\^SGAUTH:\\s*(\\d+),(\\d+),?\"?([a-zA-Z0-9_-]+)?\"?", 0, 0, NULL);
+    g_assert (r != NULL);
+
+    g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, NULL);
+    while (g_match_info_matches (match_info)) {
+        guint sgauth_cid = 0;
+
+        if (mm_get_uint_from_match_info (match_info, 1, &sgauth_cid) &&
+            (sgauth_cid == cid)) {
+            guint cinterion_auth_type = 0;
+
+            mm_get_uint_from_match_info (match_info, 2, &cinterion_auth_type);
+            *out_auth = mm_auth_type_from_cinterion_auth_type (cinterion_auth_type);
+            *out_username = mm_get_string_unquoted_from_match_info (match_info, 3);
+            return TRUE;
+        }
+        g_match_info_next (match_info, NULL);
+    }
+
+    g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND,
+                 "Auth settings for context %u not found", cid);
+    return FALSE;
+}
+
 /*****************************************************************************/
 /* ^SMONG response parser */
 
-static MMModemAccessTechnology
-get_access_technology_from_smong_gprs_status (guint    gprs_status,
-                                              GError **error)
+static gboolean
+get_access_technology_from_smong_gprs_status (guint                     gprs_status,
+                                              MMModemAccessTechnology  *out,
+                                              GError                  **error)
 {
     switch (gprs_status) {
     case 0:
-        return MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
+        *out = MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
+        return TRUE;
     case 1:
     case 2:
-        return MM_MODEM_ACCESS_TECHNOLOGY_GPRS;
+        *out = MM_MODEM_ACCESS_TECHNOLOGY_GPRS;
+        return TRUE;
     case 3:
     case 4:
-        return MM_MODEM_ACCESS_TECHNOLOGY_EDGE;
+        *out = MM_MODEM_ACCESS_TECHNOLOGY_EDGE;
+        return TRUE;
     default:
         break;
     }
@@ -871,7 +967,7 @@ get_access_technology_from_smong_gprs_status (guint    gprs_status,
                  "Couldn't get network capabilities, "
                  "unsupported GPRS status value: '%u'",
                  gprs_status);
-    return MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
+    return FALSE;
 }
 
 gboolean
@@ -879,9 +975,10 @@ mm_cinterion_parse_smong_response (const gchar              *response,
                                    MMModemAccessTechnology  *access_tech,
                                    GError                  **error)
 {
-    GError     *inner_error = NULL;
-    GMatchInfo *match_info = NULL;
-    GRegex     *regex;
+    guint                  value = 0;
+    GError                *inner_error = NULL;
+    g_autoptr(GMatchInfo)  match_info = NULL;
+    g_autoptr(GRegex)      regex = NULL;
 
     /* The AT^SMONG command returns a cell info table, where the second
      * column identifies the "GPRS status", which is exactly what we want.
@@ -900,26 +997,21 @@ mm_cinterion_parse_smong_response (const gchar              *response,
                          0, NULL);
     g_assert (regex);
 
-    if (g_regex_match_full (regex, response, strlen (response), 0, 0, &match_info, &inner_error)) {
-        guint value = 0;
-
-        if (!mm_get_uint_from_match_info (match_info, 2, &value))
-            inner_error = g_error_new (MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
-                                       "Couldn't read 'GPRS status' field from AT^SMONG response");
-        else if (access_tech)
-            *access_tech = get_access_technology_from_smong_gprs_status (value, &inner_error);
-    }
-
-    g_match_info_free (match_info);
-    g_regex_unref (regex);
+    g_regex_match_full (regex, response, strlen (response), 0, 0, &match_info, &inner_error);
 
     if (inner_error) {
+        g_prefix_error (&inner_error, "Failed to match AT^SMONG response: ");
         g_propagate_error (error, inner_error);
         return FALSE;
     }
 
-    g_assert (access_tech != MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN);
-    return TRUE;
+    if (!g_match_info_matches (match_info) || !mm_get_uint_from_match_info (match_info, 2, &value)) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Couldn't read 'GPRS status' field from AT^SMONG response");
+        return FALSE;
+    }
+
+    return get_access_technology_from_smong_gprs_status (value, access_tech, error);
 }
 
 /*****************************************************************************/
@@ -1398,3 +1490,183 @@ mm_cinterion_smoni_response_to_signal_info (const gchar  *response,
 
     return TRUE;
 }
+
+/*****************************************************************************/
+/* provider cfg information to CID number for EPS initial settings */
+
+/*
+ * at^scfg="MEopMode/Prov/Cfg"
+ * ^SCFG: "MEopMode/Prov/Cfg","vdfde"
+ * ^SCFG: "MEopMode/Prov/Cfg","attus"
+ * ^SCFG: "MEopMode/Prov/Cfg","2"      -> PLS8-X vzw
+ * ^SCFG: "MEopMode/Prov/Cfg","vzwdcus" -> PLAS9-x vzw
+ * ^SCFG: "MEopMode/Prov/Cfg","tmode" -> t-mob germany
+ * OK
+ */
+gboolean
+mm_cinterion_provcfg_response_to_cid (const gchar             *response,
+                                      MMCinterionModemFamily   modem_family,
+                                      MMModemCharset           charset,
+                                      gpointer                 log_object,
+                                      gint                    *cid,
+                                      GError                 **error)
+{
+    g_autoptr(GRegex)      r = NULL;
+    g_autoptr(GMatchInfo)  match_info = NULL;
+    g_autofree gchar      *mno = NULL;
+    GError                *inner_error = NULL;
+
+    r = g_regex_new ("\\^SCFG:\\s*\"MEopMode/Prov/Cfg\",\\s*\"([0-9a-zA-Z*]*)\"", 0, 0, NULL);
+    g_assert (r != NULL);
+
+    g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, &inner_error);
+
+    if (inner_error) {
+        g_prefix_error (&inner_error, "Failed to match Prov/Cfg response: ");
+        g_propagate_error (error, inner_error);
+        return FALSE;
+    }
+
+    if (!g_match_info_matches (match_info)) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Couldn't match Prov/Cfg response");
+        return FALSE;
+    }
+
+    mno = mm_get_string_unquoted_from_match_info (match_info, 1);
+    if (mno && modem_family == MM_CINTERION_MODEM_FAMILY_IMT) {
+        gchar *mno_utf8;
+
+        mno_utf8 = mm_modem_charset_str_to_utf8 (mno, -1, charset, FALSE, error);
+        if (!mno_utf8)
+            return FALSE;
+        g_free (mno);
+        mno = mno_utf8;
+    }
+    mm_obj_dbg (log_object, "current mno: %s", mno ? mno : "none");
+
+    /* for Cinterion LTE modules, some CID numbers have special meaning.
+     * This is dictated by the chipset and by the MNO:
+     * - the chipset uses a special one, CID 1, as a LTE combined attach chipset
+     * - the MNOs can define the sequence and number of APN to be used for their network.
+     *   This takes priority over the chipset preferences, and therefore for some of them
+     *   the CID for the initial EPS context must be changed.
+     */
+    if (g_strcmp0 (mno, "2") == 0 || g_strcmp0 (mno, "vzwdcus") == 0)
+        *cid = 3;
+    else if (g_strcmp0 (mno, "tmode") == 0)
+        *cid = 2;
+    else
+        *cid = 1;
+    return TRUE;
+}
+
+/*****************************************************************************/
+/* Auth related helpers */
+
+typedef enum {
+    BEARER_CINTERION_AUTH_UNKNOWN   = -1,
+    BEARER_CINTERION_AUTH_NONE      =  0,
+    BEARER_CINTERION_AUTH_PAP       =  1,
+    BEARER_CINTERION_AUTH_CHAP      =  2,
+    BEARER_CINTERION_AUTH_MSCHAPV2  =  3,
+} BearerCinterionAuthType;
+
+static BearerCinterionAuthType
+parse_auth_type (MMBearerAllowedAuth mm_auth)
+{
+    switch (mm_auth) {
+    case MM_BEARER_ALLOWED_AUTH_NONE:
+        return BEARER_CINTERION_AUTH_NONE;
+    case MM_BEARER_ALLOWED_AUTH_PAP:
+        return BEARER_CINTERION_AUTH_PAP;
+    case MM_BEARER_ALLOWED_AUTH_CHAP:
+        return BEARER_CINTERION_AUTH_CHAP;
+    case MM_BEARER_ALLOWED_AUTH_MSCHAPV2:
+        return BEARER_CINTERION_AUTH_MSCHAPV2;
+    case MM_BEARER_ALLOWED_AUTH_UNKNOWN:
+    case MM_BEARER_ALLOWED_AUTH_MSCHAP:
+    case MM_BEARER_ALLOWED_AUTH_EAP:
+    default:
+        return BEARER_CINTERION_AUTH_UNKNOWN;
+    }
+}
+
+MMBearerAllowedAuth
+mm_auth_type_from_cinterion_auth_type (guint cinterion_auth)
+{
+    switch (cinterion_auth) {
+    case BEARER_CINTERION_AUTH_NONE:
+        return MM_BEARER_ALLOWED_AUTH_NONE;
+    case BEARER_CINTERION_AUTH_PAP:
+        return MM_BEARER_ALLOWED_AUTH_PAP;
+    case BEARER_CINTERION_AUTH_CHAP:
+        return MM_BEARER_ALLOWED_AUTH_CHAP;
+    default:
+        return MM_BEARER_ALLOWED_AUTH_UNKNOWN;
+    }
+}
+
+/* Cinterion authentication is done with the command AT^SGAUTH,
+   whose syntax depends on the modem family, as follow:
+   - AT^SGAUTH=<cid>[, <auth_type>[, <user>, <passwd>]] for the IMT family
+   - AT^SGAUTH=<cid>[, <auth_type>[, <passwd>, <user>]] for the rest */
+gchar *
+mm_cinterion_build_auth_string (gpointer                log_object,
+                                MMCinterionModemFamily  modem_family,
+                                MMBearerProperties     *config,
+                                guint                   cid)
+{
+    MMBearerAllowedAuth      auth;
+    BearerCinterionAuthType  encoded_auth = BEARER_CINTERION_AUTH_UNKNOWN;
+    gboolean                 has_user;
+    gboolean                 has_passwd;
+    const gchar             *user;
+    const gchar             *passwd;
+    g_autofree gchar        *quoted_user = NULL;
+    g_autofree gchar        *quoted_passwd = NULL;
+
+    user   = mm_bearer_properties_get_user         (config);
+    passwd = mm_bearer_properties_get_password     (config);
+    auth   = mm_bearer_properties_get_allowed_auth (config);
+
+    has_user     = (user   && user[0]);
+    has_passwd   = (passwd && passwd[0]);
+    encoded_auth = parse_auth_type (auth);
+
+    /* When 'none' requested, we won't require user/password */
+    if (encoded_auth == BEARER_CINTERION_AUTH_NONE) {
+        if (has_user || has_passwd)
+            mm_obj_warn (log_object, "APN user/password given but 'none' authentication requested");
+        if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT)
+            return g_strdup_printf ("^SGAUTH=%u,%d,\"\",\"\"", cid, encoded_auth);
+        return g_strdup_printf ("^SGAUTH=%u,%d", cid, encoded_auth);
+    }
+
+    /* No explicit auth type requested? */
+    if (encoded_auth == BEARER_CINTERION_AUTH_UNKNOWN) {
+        /* If no user/passwd given, do nothing */
+        if (!has_user && !has_passwd)
+            return NULL;
+
+        /* If user/passwd given, default to CHAP (more common than PAP) */
+        mm_obj_dbg (log_object, "APN user/password given but no authentication type explicitly requested: defaulting to 'CHAP'");
+        encoded_auth = BEARER_CINTERION_AUTH_CHAP;
+    }
+
+    quoted_user   = mm_port_serial_at_quote_string (user   ? user   : "");
+    quoted_passwd = mm_port_serial_at_quote_string (passwd ? passwd : "");
+
+    if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT)
+        return g_strdup_printf ("^SGAUTH=%u,%d,%s,%s",
+                                cid,
+                                encoded_auth,
+                                quoted_user,
+                                quoted_passwd);
+
+    return g_strdup_printf ("^SGAUTH=%u,%d,%s,%s",
+                            cid,
+                            encoded_auth,
+                            quoted_passwd,
+                            quoted_user);
+}
