@@ -143,6 +143,25 @@ mm_shared_quectel_firmware_load_update_settings_finish (MMIfaceModemFirmware  *s
     return g_task_propagate_pointer (G_TASK (res), error);
 }
 
+static void
+quectel_get_firmware_version_ready (MMBaseModem  *modem,
+                                    GAsyncResult *res,
+                                    GTask        *task)
+{
+    MMFirmwareUpdateSettings *update_settings;
+    const gchar              *version;
+
+    update_settings = g_task_get_task_data (task);
+
+    /* Set full firmware version */
+    version = mm_base_modem_at_command_finish (modem, res, NULL);
+    if (version)
+        mm_firmware_update_settings_set_version (update_settings, version);
+
+    g_task_return_pointer (task, g_object_ref (update_settings), g_object_unref);
+    g_object_unref (task);
+}
+
 static void
 qfastboot_test_ready (MMBaseModem  *self,
                       GAsyncResult *res,
@@ -150,15 +169,65 @@ qfastboot_test_ready (MMBaseModem  *self,
 {
     MMFirmwareUpdateSettings *update_settings;
 
-    if (!mm_base_modem_at_command_finish (self, res, NULL))
-        update_settings = mm_firmware_update_settings_new (MM_MODEM_FIRMWARE_UPDATE_METHOD_NONE);
-    else {
-        update_settings = mm_firmware_update_settings_new (MM_MODEM_FIRMWARE_UPDATE_METHOD_FASTBOOT);
+    update_settings = g_task_get_task_data (task);
+
+    /* Set update method */
+    if (mm_base_modem_at_command_finish (self, res, NULL)) {
+        mm_firmware_update_settings_set_method (update_settings, MM_MODEM_FIRMWARE_UPDATE_METHOD_FASTBOOT);
         mm_firmware_update_settings_set_fastboot_at (update_settings, "AT+QFASTBOOT");
+    } else
+        mm_firmware_update_settings_set_method (update_settings, MM_MODEM_FIRMWARE_UPDATE_METHOD_NONE);
+
+    /* Fetch full firmware info */
+    mm_base_modem_at_command (MM_BASE_MODEM (self),
+                              "+QGMR?",
+                              3,
+                              FALSE,
+                              (GAsyncReadyCallback) quectel_get_firmware_version_ready,
+                              task);
+}
+
+static void
+quectel_at_port_get_firmware_revision_ready (MMBaseModem  *self,
+                                             GAsyncResult *res,
+                                             GTask        *task)
+{
+    MMFirmwareUpdateSettings    *update_settings;
+    const gchar                 *revision;
+    const gchar                 *name;
+    const gchar                 *id;
+    g_autoptr(GPtrArray)         ids = NULL;
+    GError                      *error = NULL;
+
+    update_settings = g_task_get_task_data (task);
+
+    /* Set device ids */
+    ids = mm_iface_firmware_build_generic_device_ids (MM_IFACE_MODEM_FIRMWARE (self), &error);
+    if (error) {
+        mm_obj_warn (self, "failed to build generic device ids: %s", error->message);
+        g_task_return_error (task, error);
+        g_object_unref (task);
+        return;
     }
 
-    g_task_return_pointer (task, update_settings, g_object_unref);
-    g_object_unref (task);
+    /* Add device id based on modem name */
+    revision = mm_base_modem_at_command_finish (self, res, NULL);
+    if (revision && g_utf8_validate (revision, -1, NULL)) {
+        name = g_strndup (revision, 7);
+        mm_obj_dbg (self, "revision %s converted to modem name %s", revision, name);
+        id = (const gchar *) g_ptr_array_index (ids, 0);
+        g_ptr_array_insert (ids, 0, g_strdup_printf ("%s&NAME_%s", id, name));
+    }
+
+    mm_firmware_update_settings_set_device_ids (update_settings, (const gchar **)ids->pdata);
+
+    /* Check fastboot support */
+    mm_base_modem_at_command (self,
+                              "AT+QFASTBOOT=?",
+                              3,
+                              TRUE,
+                              (GAsyncReadyCallback) qfastboot_test_ready,
+                              task);
 }
 
 void
@@ -167,14 +236,32 @@ mm_shared_quectel_firmware_load_update_settings (MMIfaceModemFirmware *self,
                                                  gpointer              user_data)
 {
     GTask *task;
+    MMPortSerialAt *at_port;
+    MMFirmwareUpdateSettings *update_settings;
 
     task = g_task_new (self, NULL, callback, user_data);
-    mm_base_modem_at_command (MM_BASE_MODEM (self),
-                              "AT+QFASTBOOT=?",
-                              3,
-                              TRUE,
-                              (GAsyncReadyCallback)qfastboot_test_ready,
-                              task);
+
+    at_port = mm_base_modem_peek_best_at_port (MM_BASE_MODEM (self), NULL);
+    if (at_port) {
+        update_settings = mm_firmware_update_settings_new (MM_MODEM_FIRMWARE_UPDATE_METHOD_NONE);
+        g_task_set_task_data (task, update_settings, g_object_unref);
+
+        /* Fetch modem name */
+        mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                  "+CGMR",
+                                  3,
+                                  TRUE,
+                                  (GAsyncReadyCallback) quectel_at_port_get_firmware_revision_ready,
+                                  task);
+
+        return;
+    }
+
+    g_task_return_new_error (task,
+                             MM_CORE_ERROR,
+                             MM_CORE_ERROR_FAILED,
+                             "Couldn't find a port to fetch firmware info");
+    g_object_unref (task);
 }
 
 /*****************************************************************************/
@@ -431,10 +518,13 @@ mm_shared_quectel_location_load_capabilities (MMIfaceModemLocation *_self,
  *  1) "+QGPSCFG=\"nmeasrc\",1" will be necessary for getting location data
  *     without the nmea port.
  *  2) may be necessary to set "+QGPSCFG=\"gpsnmeatype\".
+ *  3) QGPSXTRA=1 is necessary to support XTRA assistance data for
+ *     faster GNSS location locks.
  */
 static const MMBaseModemAtCommand gps_startup[] = {
     { "+QGPSCFG=\"outport\",\"usbnmea\"", 3, FALSE, mm_base_modem_response_processor_no_result_continue },
     { "+QGPS=1",                          3, FALSE, mm_base_modem_response_processor_no_result_continue },
+    { "+QGPSXTRA=1",                      3, FALSE, mm_base_modem_response_processor_no_result_continue },
     { NULL }
 };
 
@@ -591,7 +681,7 @@ disable_location_gathering_parent_ready (MMIfaceModemLocation *self,
                                          GAsyncResult         *res,
                                          GTask                *task)
 {
-    GError  *error;
+    GError  *error = NULL;
     Private *priv;
 
     priv = get_private (MM_SHARED_QUECTEL (self));
