@@ -31,7 +31,6 @@
 #include "mm-broadband-modem.h"
 #include "mm-broadband-modem-thuraya.h"
 #include "mm-private-boxed-types.h"
-#include "mm-log.h"
 
 G_DEFINE_TYPE (MMPluginThuraya, mm_plugin_thuraya, MM_TYPE_PLUGIN)
 
@@ -64,10 +63,10 @@ mm_plugin_create (void)
 
     return MM_PLUGIN (
         g_object_new (MM_TYPE_PLUGIN_THURAYA,
-                      MM_PLUGIN_NAME,                    "Thuraya",
-                      MM_PLUGIN_ALLOWED_SUBSYSTEMS,      subsystems,
-                      MM_PLUGIN_ALLOWED_VENDOR_IDS,      vendor_ids,
-                      MM_PLUGIN_ALLOWED_AT,              TRUE,
+                      MM_PLUGIN_NAME,               MM_MODULE_NAME,
+                      MM_PLUGIN_ALLOWED_SUBSYSTEMS, subsystems,
+                      MM_PLUGIN_ALLOWED_VENDOR_IDS, vendor_ids,
+                      MM_PLUGIN_ALLOWED_AT,         TRUE,
                       NULL));
 }
 
