@@ -96,31 +96,55 @@ struct _MMDevicePrivate {
 
 /*****************************************************************************/
 
+static MMPortProbe *
+probe_list_lookup_by_device (GList          *port_probes,
+                             MMKernelDevice *kernel_port)
+{
+    GList *l;
+
+    for (l = port_probes; l; l = g_list_next (l)) {
+        MMPortProbe *probe = MM_PORT_PROBE (l->data);
+
+        if (mm_kernel_device_cmp (mm_port_probe_peek_port (probe), kernel_port))
+            return probe;
+    }
+    return NULL;
+}
+
+static MMPortProbe *
+probe_list_lookup_by_name (GList       *port_probes,
+                           const gchar *subsystem,
+                           const gchar *name)
+{
+    GList *l;
+
+    for (l = port_probes; l; l = g_list_next (l)) {
+        MMPortProbe    *probe = MM_PORT_PROBE (l->data);
+        MMKernelDevice *probe_device;
+
+        probe_device = mm_port_probe_peek_port (probe);
+        if ((g_strcmp0 (subsystem, mm_kernel_device_get_subsystem (probe_device)) == 0) &&
+            (g_strcmp0 (name,      mm_kernel_device_get_name      (probe_device)) == 0))
+            return probe;
+    }
+    return NULL;
+}
+
 static MMPortProbe *
 device_find_probe_with_device (MMDevice       *self,
                                MMKernelDevice *kernel_port,
                                gboolean        lookup_ignored)
 {
-    GList *l;
+    MMPortProbe *probe;
 
-    for (l = self->priv->port_probes; l; l = g_list_next (l)) {
-        MMPortProbe *probe = MM_PORT_PROBE (l->data);
-
-        if (mm_kernel_device_cmp (mm_port_probe_peek_port (probe), kernel_port))
-            return probe;
-    }
+    probe = probe_list_lookup_by_device (self->priv->port_probes, kernel_port);
+    if (probe)
+        return probe;
 
     if (!lookup_ignored)
         return NULL;
 
-    for (l = self->priv->ignored_port_probes; l; l = g_list_next (l)) {
-        MMPortProbe *probe = MM_PORT_PROBE (l->data);
-
-        if (mm_kernel_device_cmp (mm_port_probe_peek_port (probe), kernel_port))
-            return probe;
-    }
-
-    return NULL;
+    return probe_list_lookup_by_device (self->priv->ignored_port_probes, kernel_port);
 }
 
 gboolean
@@ -130,6 +154,28 @@ mm_device_owns_port (MMDevice       *self,
     return !!device_find_probe_with_device (self, kernel_port, TRUE);
 }
 
+static MMPortProbe *
+device_find_probe_with_name (MMDevice    *self,
+                             const gchar *subsystem,
+                             const gchar *name)
+{
+    MMPortProbe *probe;
+
+    probe = probe_list_lookup_by_name (self->priv->port_probes, subsystem, name);
+    if (probe)
+        return probe;
+
+    return probe_list_lookup_by_name (self->priv->ignored_port_probes, subsystem, name);
+}
+
+gboolean
+mm_device_owns_port_name (MMDevice    *self,
+                          const gchar *subsystem,
+                          const gchar *name)
+{
+    return !!device_find_probe_with_name (self, subsystem, name);
+}
+
 static void
 add_port_driver (MMDevice       *self,
                  MMKernelDevice *kernel_port)
@@ -165,11 +211,28 @@ void
 mm_device_grab_port (MMDevice       *self,
                      MMKernelDevice *kernel_port)
 {
-    MMPortProbe *probe;
+    MMPortProbe    *probe;
+    MMKernelDevice *lower_port;
 
     if (mm_device_owns_port (self, kernel_port))
         return;
 
+    lower_port = mm_kernel_device_peek_lower_device (kernel_port);
+    if (lower_port) {
+        g_autoptr(GError) error = NULL;
+
+        /* No port probing done, at this point this is not something we require
+         * as all the virtual instantiated ports are net devices. We also avoid
+         * emitting the PORT_GRABBED signal in the MMDevice, because that is
+         * exclusively linked to a port being added to the list of probes, which
+         * we don't do here. */
+        if (self->priv->modem && !mm_base_modem_grab_link_port (self->priv->modem, kernel_port, &error))
+            mm_obj_dbg (self, "fully ignoring link port %s from now on: %s",
+                        mm_kernel_device_get_name (kernel_port),
+                        error->message);
+        return;
+    }
+
     /* Get the vendor/product IDs out of the first one that gives us
      * some valid value (it seems we may get NULL reported for VID in QMI
      * ports, e.g. Huawei E367) */
@@ -190,12 +253,19 @@ mm_device_grab_port (MMDevice       *self,
 }
 
 void
-mm_device_release_port (MMDevice       *self,
-                        MMKernelDevice *kernel_port)
+mm_device_release_port_name (MMDevice    *self,
+                             const gchar *subsystem,
+                             const gchar *name)
 {
     MMPortProbe *probe;
 
-    probe = device_find_probe_with_device (self, kernel_port, TRUE);
+    /* If modem exists, try to remove it as a link port. We also avoid emitting
+     * the PORT_RELEASED signal in this case, as the link ports are not associated
+     * to the port probe list */
+    if (self->priv->modem && mm_base_modem_release_link_port (self->priv->modem, subsystem, name, NULL))
+        return;
+
+    probe = device_find_probe_with_name (self, subsystem, name);
     if (probe) {
         /* Found, remove from lists and destroy probe */
         if (g_list_find (self->priv->port_probes, probe))
@@ -336,6 +406,9 @@ reprobe (MMDevice *self)
 {
     GError *error = NULL;
 
+    self->priv->reprobe_id = 0;
+
+    mm_obj_dbg (self, "Reprobing modem...");
     if (!mm_device_create_modem (self, &error)) {
         mm_obj_warn (self, "could not recreate modem: %s", error->message);
         g_error_free (error);
@@ -463,7 +536,7 @@ GObject *
 mm_device_get_plugin (MMDevice *self)
 {
     return (self->priv->plugin ?
-            g_object_ref (self->priv->plugin) :
+            G_OBJECT (g_object_ref (self->priv->plugin)) :
             NULL);
 }
 
@@ -500,7 +573,7 @@ mm_device_get_port_probe (MMDevice       *self,
     MMPortProbe *probe;
 
     probe = device_find_probe_with_device (self, kernel_port, FALSE);
-    return (probe ? g_object_ref (probe) : NULL);
+    return (probe ? G_OBJECT (g_object_ref (probe)) : NULL);
 }
 
 GList *
